//+------------------------------------------------------------------+
//|                                                      FullSSA.mq4 |
//|                                           Copyright  2007, klot |
//|                                                     klot@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, klot"
#property link      "klot@mail.ru"


#import "libSSA.dll" 
    void fastSingular(double& a[],int n,int lag ,int s,double& b[]);
#import


#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Yellow
#property indicator_width1 3
#property indicator_level1 0.0
#property indicator_levelstyle STYLE_DASH
#property indicator_levelcolor Magenta




extern string  note_TimeFrames = "M1;5,15,30,60H1;240H4;1440D1;10080W1;43200MN|0-CurrentTF";
extern string  TimeFrame       = "Current time frame";
extern int     Lag             = 10;
extern int     NumComps        = 2;
extern  int    PeriodNorm      = 10;
extern int     N               = 1000;
extern bool    Interpolate     = true;

   



double SSA[];
double ssaWork[];
double arryTimeSeries[];


string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;


int init()
  {

   
   SetIndexBuffer(0,SSA); SetIndexStyle(0,DRAW_LINE); SetIndexDrawBegin(0,Bars-N);
   
   
   indicatorFileName = WindowExpertName();
   calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
   returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   
   IndicatorShortName(timeFrameToString(timeFrame)+ "  Full SSA normalize correct");

   return(0);
  }

int deinit(){return(0);}



int start()
  {
   
   double dev,ma;
   int    nmax, nmin;
   int    size = N;
   
   
   if (size>Bars) size = Bars;
   if (ArraySize(arryTimeSeries) != size)
   {
   ArrayResize(arryTimeSeries,size);
   ArrayResize(ssaWork,size);
   }
   int    limit, i; 
   int    counted_bars=IndicatorCounted();
   
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
   limit=MathMin(Bars-counted_bars,Bars-1);
   if (returnBars)  { SSA[0] = limit+1; return(0); }   
           
   if (calculateValue || timeFrame==Period())
   {
      
   for( i=size-1; i>=0; i--) 
   {
   ma=iMA(NULL,0,PeriodNorm,0,MODE_SMA,PRICE_CLOSE,i);
   dev=3*iStdDev(NULL,0,PeriodNorm,0,MODE_SMA,PRICE_CLOSE,i);
   if(dev==0) dev=0.1;
   arryTimeSeries[i]=(Close[i]-ma)/dev;//iDeMarker(NULL,0,PeriodNorm,i);
   ObjectDelete("Sell"+Time[i]);
   ObjectDelete("Buy"+Time[i]);
   }
   fastSingular(arryTimeSeries,size,Lag,NumComps,ssaWork);
   ArrayCopy(SSA,ssaWork);
   
   nmax=ArrayMaximum(SSA,3,1);
   nmin=ArrayMinimum(SSA,3,1);
   if(nmax==2) {
            ObjectCreate("Sell"+Time[0],OBJ_ARROW,0,Time[0],Open[0]);
            ObjectSet("Sell"+Time[0],OBJPROP_ARROWCODE,226);
   }
   if(nmin==2) {
            ObjectCreate("Buy"+Time[0],OBJ_ARROW,0,Time[0],Open[0]);
            ObjectSet("Buy"+Time[0],OBJPROP_ARROWCODE,225);
   }
//----
   return(0);
  }
  
  
  limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"","returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
   int y = iBarShift(NULL,timeFrame,Time[i]);
   SSA[i] = iCustom(NULL,timeFrame,indicatorFileName,"","calculateValue",Lag,NumComps,PeriodNorm,N,0,y);


   if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
      if (!Interpolate) continue;
   
      datetime time = iTime(NULL,timeFrame,y);
      for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         double factor = 1.0 / n;
         for(int x = 1; x < n; x++)
         {
            SSA[i+x]= x*factor*SSA[i+n] + (1.0-x*factor)*SSA[i]; 
      
         }            
   }
   
   return(0);
               
}

//+------------------------------------------------------------------+


string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}